#!/bin/bash
# We're executing some admin task as root (modifing perms)
CLUSTER_HOME=/var/opt/jfrog/cluster

function checkEnv {
    local envVarName=$1
    eval local envVarValue=\$$envVarName
    if [ -z "$envVarValue" ]; then
        echo "Env variable $envVarName is not set, cannot start primary instance."
        exit 1
    fi
}

function checkAllEnvs {
    checkEnv ART_LICENSES
    checkEnv ART_DATABASE_CONNECTION_STRING
    checkEnv ART_DATABASE_USER
    checkEnv ART_DATABASE_PASSWORD
}

function setLicense {
    echo -n "$ART_LICENSES" | cut -d, -f1 > /etc/opt/jfrog/artifactory/artifactory.lic
}

function checkFolderPermissions {
    DIR=$1

    STAT=( $(stat -Lc "%A %G %U" $1) )
    PERM=${STAT[0]}
    GROUP=${STAT[1]}
    USER=${STAT[2]}

    if [[ $USER != "artifactory" ]] || [[ $GROUP != "artifactory"  ]] ; then
            if [[ ! $PERM =~ d......rw.  ]] ; then
                    echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions a+wr"
                    chmod a+wr $1
            fi
    elif [[ $USER == "artifactory" ]] && [[ ! $PERM =~ drw....... ]] ; then
            echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions u+wr"
            chmod u+wr $1
    elif [[ $GROUP == "artifactory" ]] && [[ ! $PERM =~ d...rw.... ]] ; then
            echo "[WARN] artifactory user doesn't have read & write permissions on $1, adding permissions g+wr"
            chmod g+wr $1
    fi
}

function configureDatabase {
    # Copy the mysql template if the file doesn't exist
    cp -n $ARTIFACTORY_HOME/misc/db/mysql.properties $CLUSTER_HOME/ha-etc/storage.properties
    # Replace url= with provided connection string...
    # Espace sed special chars & \ , 
    # ... before giving it to sed... (yes I know :|)
    local artDatabaseConnectionEscaped=$(echo $ART_DATABASE_CONNECTION_STRING | sed 's,[&\\,],\\&,g')
    local artUserNameEscaped=$(echo $ART_DATABASE_USER | sed 's,[&\\,],\\&,g')
    local artPasswordEscaped=$(echo $ART_DATABASE_PASSWORD | sed 's,[&\\,],\\&,g')
    sed -i -r -e "s,^url=.*$,url=$artDatabaseConnectionEscaped,g" $CLUSTER_HOME/ha-etc/storage.properties
    sed -i -r -e "s,^username=.*$,username=$artUserNameEscaped,g" $CLUSTER_HOME/ha-etc/storage.properties
    sed -i -r -e "s,^password=.*$,password=$artPasswordEscaped,g" $CLUSTER_HOME/ha-etc/storage.properties
    chown artifactory: $CLUSTER_HOME/ha-etc/storage.properties
}

function setPorts {
    # Artifactory's membership port, the PORT0 et PORT1 are specific to MESOS
    : ${MEMBERSHIP_PORT:=10042}
    : ${INSTANCE_PORT:=8081}
    if [ ! -z "$PORT0" ]; then
        INSTANCE_PORT=$PORT0
    fi
    if [ ! -z "$PORT1" ]; then
        MEMBERSHIP_PORT=$PORT1
    fi

    # Change the port
    sed -i -e "s,8081,$INSTANCE_PORT,g" /opt/jfrog/artifactory/tomcat/conf/server.xml
}

function setHaNodeProperties {
    cat >/etc/opt/jfrog/artifactory/ha-node.properties <<EOF
node.id="art-primary"
cluster.home=$CLUSTER_HOME
primary=true
context.url=http://$(hostname -I|awk '{print $1}'):$INSTANCE_PORT/artifactory
membership.port=$MEMBERSHIP_PORT
EOF
    chown artifactory: /etc/opt/jfrog/artifactory/ha-node.properties
}

function setClusterHome {
    # Checking permissions for mounted volumes
    local artifactoryFolders=$(mount | grep artifactory | awk '{print $3}')
    for folder in $artifactoryFolders ; do
        checkFolderPermissions $folder
    done
    checkFolderPermissions $CLUSTER_HOME

    # Create the cluster home content the first time 
    if [ ! -f $CLUSTER_HOME/ha-etc/cluster.properties ]; then
        su artifactory -c "mkdir -p $CLUSTER_HOME/{ha-etc/{UI,plugins},ha-data/{filestore,tmp},ha-backup}"
        su artifactory -c "echo security.token=$(uuidgen) >$CLUSTER_HOME/ha-etc/cluster.properties"
    fi

    # Copy the licensesBucket plugin into the cluster home (always do it in case of an update of the plugin in the image)
    su artifactory -c "cp -f /var/opt/jfrog/artifactory/etc/plugins/licensesBucket.groovy $CLUSTER_HOME/ha-etc/plugins/licensesBucket.groovy"
}

function waitForMysql {
   # Extract mysql hostname and port from the jdbc url
    local mysqlHostAndPort=$(echo $ART_DATABASE_CONNECTION_STRING | sed -e 's,^jdbc:mysql:\/\/\(.*\)$,\1:,g' | cut -d/ -f1)
    while ! curl -s "$mysqlHostAndPort" > /dev/null; do echo 'waiting for mysql'; sleep 3; done 
}

function setInitialConfiguration {
    # Artifactory's external server name
    : ${ART_SERVER_NAME:=artifactory-cluster}
    # Artifactory's port method, default to PORTPERREPO (can be SUBDOMAIN)
    : ${ART_REVERSE_PROXY_METHOD:=portPerRepo}
    if [ -f /tmp/artifactory.config.xml ]; then
        sed -i -e "s,\[SERVERNAME\],$ART_SERVER_NAME,g" /tmp/artifactory.config.xml
        sed -i -e "s,\[RPMETHOD\],$ART_REVERSE_PROXY_METHOD,g" /tmp/artifactory.config.xml
        sed -i -e "s,\[PORT\],$INSTANCE_PORT,g" /tmp/artifactory.config.xml
        mv /tmp/artifactory.config.xml $CLUSTER_HOME/ha-etc/artifactory.config.xml
    fi
}

checkAllEnvs
setLicense
setPorts
setClusterHome
setInitialConfiguration
configureDatabase
setHaNodeProperties

# Wait and run as artifactory user
waitForMysql

exec su artifactory -m /opt/jfrog/artifactory/bin/artifactory.sh
